// -*- C++ -*-
/*!
 * @file  pathProvider.h
 * @brief Module providing path
 * @date  $Date$
 *
 * $Id$
 */

#ifndef PATHPROVIDER_H
#define PATHPROVIDER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

#include "PlannerData.hh"

#include <deque>
#include <fstream>

using namespace RTC;

/*!
 * @class pathProvider
 * @brief Module providing path
 *
 */
class pathProvider
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  pathProvider(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~pathProvider();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /*!
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 
   * - Name: ${configParam.docDataName} CommandDegFlag
   * - DefaultValue: 0
   */
  bool m_CommandDegFlag;
  /*!
   * 
   * - Name: ${configParam.docDataName} CommandOffsetFile
   * - DefaultValue: none
   */
  std::string m_CommandOffsetFile;
  /*!
   * 
   * - Name: ${configParam.docDataName} PathFile
   * - DefaultValue: none
   */
  std::string m_PathFile;
  /*!
   * 
   * - Name: ${configParam.docDataName} InTimeStep
   * - DefaultValue: 0.05
   */
  double m_InTimeStep;
  /*!
   * 
   * - Name: ${configParam.docDataName} OutTimeStep
   * - DefaultValue: 0.01
   */
  double m_OutTimeStep;
  /*!
   * 
   * - Name: ${configParam.docDataName} MinDist
   * - DefaultValue: 0.1
   */
  double m_MinDist;
  /*!
   * 
   * - Name: ${configParam.docDataName} OutputMode
   * - DefaultValue: const
   */
  std::string m_OutputMode;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TrajData m_inPath;
  /*!
   */
  InPort<TrajData> m_inPathIn;
  TimedDoubleSeq m_inCurrentConfig;
  /*!
   */
  InPort<TimedDoubleSeq> m_inCurrentConfigIn;
  TimedBoolean m_PathStart;
  /*!
   */
  InPort<TimedBoolean> m_PathStartIn;
  TimedLong m_CrntBufSize;
  /*!
   */
  InPort<TimedLong> m_CrntBufSizeIn;
  TimedBoolean m_inStop;
  /*!
   */
  InPort<TimedBoolean> m_inStopIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_outCommandConfig;
  /*!
   */
  OutPort<TimedDoubleSeq> m_outCommandConfigOut;
  TrajData m_outCommandConfigSeq;
  /*!
   */
  OutPort<TrajData> m_outCommandConfigSeqOut;
  TimedDoubleSeq m_outCurrentConfig;
  /*!
   */
  OutPort<TimedDoubleSeq> m_outCurrentConfigOut;
  TimedBoolean m_outMotionFinished;
  /*!
   */
  OutPort<TimedBoolean> m_outMotionFinishedOut;
  TimedBoolean m_outStop;
  /*!
   */
  OutPort<TimedBoolean> m_outStopOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

  unsigned long m_crntPathRank;

  // queue of interpolated configurations by m_OutTimeStep
  std::deque<RTC::TimedDoubleSeq> m_OutConfigQueue;

  std::vector<double> m_CommandOffsetVec;

  RTC::ReturnCode_t readPath();

  RTC::ReturnCode_t readCommandOffset();

  RTC::ReturnCode_t fillConfigQueue();

  /// \brief set the time to the path read with the sampling time m_InTimeStep
  void setPathTime();

  /// \brief print the path with the time
  void printPath();

  /// \for debut
  std::ofstream m_foutSeq;

};


extern "C"
{
  void pathProviderInit(RTC::Manager* manager);
};

#endif // PATHPROVIDER_H
